import MySQLdb

class DbDriver:
	def __init__(self, db_host, db_user=None, db_password=None, db_database=None, use_cache=False, logit_class=None):
		self.dbc=MySQLdb
		self.cursor=None
		self.log=logit_class
		self.caching=use_cache
		self.cache=dict({})
		self.q=DbDriver_query(self)
		self.connect(db_host, db_user, db_password, db_database)
	
	def connect(self, db_host, db_user=None, db_password=None, db_database=None):
		try:
			if db_host=='localhost':
				self.cnx=self.dbc.connect(unix_socket="/var/run/mysqld/mysqld.sock", user=db_user, passwd=db_password, db=db_database)
			else:
				self.cnx=self.dbc.connect(host=db_host, user=db_user, passwd=db_password, db=db_database)
			self.openCursor()
		except Exception as e:
			self.logmsg('connect', 'err', e=e)
	
	def close(self):
		try:
			self.closeCursor()
			self.cnx.close()
		except:
			pass
	
	
	def openCursor(self):
		try:
			self.cursor=self.cnx.cursor(cursorclass=self.dbc.cursors.DictCursor)
		except Exception as e:
			self.logmsg('openCursor', 'err', e=e)
			self.cursor=None
			pass
	
	def closeCursor(self):
		try:
			self.cursor.close()
		except Exception as e:
			self.logmsg('closeCursor', 'warn', e=e)
			pass
		self.cursor=None
	
	
	def query(self, request, data):
		try:
			self.cursor.execute(request, data)
			ret=True
		except Exception as e:
			self.logmsg('query', 'err', e=e)
			ret=False
			pass
		return ret
	
	def commit(self):
		try:
			self.cnx.commit()
			ret=True
		except Exception as e:
			self.logmsg('commit', 'err', e=e)
			ret=False
			pass
		return ret
	
	def fetch(self, only_one_row=False):
		try:
			if only_one_row==True:
				ret=self.cursor.fetchone()
			else:
				ret=[]
				result=self.cursor.fetchone()
				while result is not None:
					ret.append(result)
					result=self.cursor.fetchone()
		except Exception as e:
			self.logmsg('fetch', 'warn', e=e)
			ret=False
			pass
		return ret
	
	
	def logmsg(self, function, severity, message=None, e=None):
		if message==None:
			message=''
		if self.log!=None:
			if e!=None:
				self.log.set(severity, 'dbmysql().'+function+'.exception('+str(type(e)).replace('<', '').replace('>', '')+'): '+str(e))
			else:
				self.log.set(severity, 'dbmysql().'+function+': '+message)


class DbDriver_query:
	def __init__(self, parent):
		self.dbd=parent
		self.new()
	
	def new(self):
		self.q=''
		self.request=dict({'funct':None, 'table':None, 'cols':None, 'values':None, 'where':None, 'order':None, 'limit':None, 'data':None})
		return True
	
	
	def select(self, table=None, cols=None, where=None, order=None, limit=None, distinct=False, data=None):
		if distinct==True:
			self.request['funct']='select distinct'
		else:
			self.request['funct']='select'
		if table is not None:
			self.table(table)
		if cols is not None:
			self.cols(cols)
		if where is not None:
			self.where(where)
		if order is not None:
			self.order(order)
		if limit is not None:
			self.limit(limit)
		if data is not None:
			self.data(data)
		return self.run()
	
	def insert(self, table=None, cols=None, values=None, data=None):
		self.request['funct']='insert'
		if table is not None:
			self.table(table)
		if cols is not None:
			self.cols(cols)
		if values is not None:
			self.values(values)
		if data is not None:
			self.data(data)
		return self.run()
	
	def update(self, table=None, cols=None, values=None, where=None, limit=None, data=None):
		self.request['funct']='update'
		if table is not None:
			self.table(table)
		if cols is not None:
			self.cols(cols)
		if values is not None:
			self.values(values)
		if where is not None:
			self.where(where)
		if limit is not None:
			self.limit(limit)
		if data is not None:
			self.data(data)
		return self.run()
	
	def delete(self, table=None, where=None, data=None):
		self.request['funct']='delete'
		if table is not None:
			self.table(table)
		if where is not None:
			self.where(where)
		if data is not None:
			self.data(data)
		return self.run()
	
	
	def table(self, table):
		self.request['table']=table
		return self.request['table']
	def cols(self, cols):
		self.request['cols']=self.parse(cols)
		return self.request['cols']
	def values(self, values):
		self.request['values']=self.parse(values)
		return self.request['values']
	def where(self, where):
		self.request['where']=where
		return self.request['where']
	def order(self, order):
		self.request['order']=self.parse(order)
		return self.request['order']
	def limit(self, cols):
		self.request['limit']=self.parse(limit)
		return self.request['limit']
	def data(self, data):
		self.request['data']=data
		return self.request['data']
	
	
	def parse(self, content):
		if isinstance(content, list)==False:
			content=[content]
		return content
	
	def check(self, items):
		ret=True
		i=0
		while i<len(items):
			if self.request[items[i]] is None:
				ret=False
			i=i+1
		return ret
	
	
	def assemble(self):
		self.q=''
		ret=True
		if self.request['funct'] is not None:
			self.q=self.request['funct']
			if self.request['funct']=='select' or self.request['funct']=='select distinct':
				ret=self.check(['table', 'cols'])
				if ret==True:
					self.q=self.q+' '+self.request['cols'].split(', ')+' from '+self.request['table']
					if self.request['where'] is not None:
						self.q=self.q+' where '+self.request['where']
					if self.request['order'] is not None:
						self.q=self.q+' order by '+self.request['order'].split(', ')
					if self.request['limit'] is not None:
						self.q=self.q+' limit '+self.request['limit'].split(', ')
			elif self.request['funct']=='insert':
				ret=self.check(['table', 'cols', 'values'])
				if ret==True:
					self.q=self.q+' into '+self.request['table']+' ('+self.request['cols'].split(', ')+') values ('+self.request['values'].split(', ')+')'
			elif self.request['funct']=='update':
				ret=self.check(['table', 'cols', 'values'])
				try:
					cvsplit=[]
					i=0
					while i<len(self.request['cols']):
						cvsplit.append(self.request['cols'][i]+'='+self.request['values'][i])
						i=i+1
				except:
					ret=False
					pass
				if ret==True:
					self.q=self.q+' '+self.request['table']+' set '+cvsplit.split(', ')
					if self.request['where'] is not None:
						self.q=self.q+' where '+self.request['where']
					if self.request['limit'] is not None:
						self.q=self.q+' limit '+self.request['limit'].split(', ')
			elif self.request['funct']=='delete':
				ret=self.check(['table'])
				if ret==True:
					self.q=self.q+' from '+self.request['table']
					if self.request['where'] is not None:
						self.q=self.q+' where '+self.request['where']
		else:
			ret=False
		return ret
	
	def run(self):
		assemb=self.assemble()
		if assemb==True:
			return self.dbd.query(self.q, self.request['data'])
		else:
			return assemb
