import time
import socket
import threading


def NetioAddress(family, target, port=0):
	if family.upper()=='INET4':
		family='INET'
	if family.upper()=='UNIX':
		addr=[socket.AF_UNIX, target]
	elif family.upper()=='INET':
		if port>0:
			addr=[socket.AF_INET, (target, port)]
		else:
			addr=None
	elif family.upper()=='INET6':
		if port>0:
			addr=[socket.AF_INET6, (target, port)]
		else:
			addr=None
	else:
		addr=None
	return addr


class NetioData:
	def __init__(self):
		self.buff=''
	
	def checkBuff(self):
		out=''
		pos=self.buff.find('<<##')
		if pos>-1:
			out=self.buff[0:pos]
			self.buff=self.buff[pos+4:]
		return out
	
	def send(self, csocket, data):
		txdata=str(data)+'<<##'
		return csocket.sendall(txdata)
	
	def recv(self, csocket, length):
		received=self.checkBuff()
		while received=='':
			self.buff=self.buff+csocket.recv(length)
			received=self.checkBuff()
		return received



class NetioServer:
	def __init__(self, sid, tx_address=None, rx_address=None, rx_callback=None, max_queued=4, length=256, timeout=None):
		self.cfg=dict({'sid':sid, 'txaddr':tx_address, 'txl':length, 'rxaddr':rx_address, 'rxl':length, 'rxc':rx_callback, 'tout':timeout})
		self.client=dict({})
		if self.cfg['rxaddr'] is not None:
			self.rxth=NetioServerTh(self.cfg['rxaddr'], callback=self.cbNewRxConn, max_queued=max_queued, timeout=self.cfg['tout'])
			self.rxth.start()
		if self.cfg['txaddr'] is not None:
			self.txth=NetioServerTh(self.cfg['txaddr'], callback=self.cbNewTxConn, max_queued=max_queued, timeout=self.cfg['tout'])
			self.txth.start()
	
	def close(self):
		clients=list(self.client.items())
		for item in clients:
			try:
				self.client[item[0]].rx.close()
			except:
				pass
			try:
				self.client[item[0]].tx.close()
			except:
				pass
		try:
			self.rxth.close()
		except:
			pass
		try:
			self.txth.close()
		except:
			pass
	
	
	def cbNewRxConn(self, client, addr):
		machine=self.identNewConn(client)
		if machine is not None:
			self.client[machine].rx=NetioRxTh(machine, client, self.cfg['rxc'], self.cfg['rxl'], self.cfg['tout'])
			self.client[machine].rx.start()
	
	def cbNewTxConn(self, client, addr):
		machine=self.identNewConn(client)
		if machine is not None:
			self.client[machine].tx=NetioTxTh(machine, client, self.cfg['txl'], self.cfg['tout'])
	
	def identNewConn(self, client):
		ret=None
		d=NetioData()
		client.settimeout(self.cfg['tout'])
		machine=str(d.recv(client, self.cfg['rxl']))
		if machine not in self.client:
			self.client.update({machine:NetioClientObj()})
		ret=machine
		return ret


class NetioClient:
	def __init__(self, cid, tx_address=None, rx_address=None, rx_callback=None, length=256, timeout=None):
		self.cfg=dict({'cid':cid, 'txaddr':tx_address, 'txl':length, 'rxaddr':rx_address, 'rxl':length, 'rxc':rx_callback, 'tout':timeout})
		if self.cfg['rxaddr'] is not None:
			self.rxth=NetioClientTh(self.cfg['rxaddr'], timeout=self.cfg['tout'])
			self.createRxConn(self.rxth.create())
		if self.cfg['txaddr'] is not None:
			self.txth=NetioClientTh(self.cfg['txaddr'], timeout=self.cfg['tout'])
			self.createTxConn(self.txth.create())
	
	def close(self):
		try:
			self.rxth.close()
		except:
			pass
		try:
			self.txth.close()
		except:
			pass
	
	def createRxConn(self, ssocket):
		ssocket.send(self.cfg['cid'])
		self.rx=NetioRxTh(self.cfg['cid'], ssocket, self.cfg['rxc'], self.cfg['rxl'], self.cfg['tout'])
		self.rx.start()
	
	def createTxConn(self, ssocket):
		d=NetioData()
		d.send(ssocket, self.cfg['cid'])
		self.tx=NetioTxTh(self.cfg['cid'], ssocket, self.cfg['rxl'], self.cfg['tout'])




class NetioServerTh(threading.Thread):
	def __init__(self, address, callback=None, max_queued=4, timeout=None):
		self.conn=address
		self.callback=callback
		self.queue=max_queued
		self.timeout=timeout
		self.islistening=False
		self.runstate=-1
		threading.Thread.__init__(self)
	
	def close(self):
		self.runstate=0
	
	def run(self):
		s=False
		while self.islistening==False:
			if self.runstate!=0:
				try:
					s=socket.socket(self.conn[0])
					s.bind(self.conn[1])
					s.settimeout(self.timeout)
					s.listen(self.queue)
					self.islistening=True
					self.runstate=1
				except:
					time.sleep(10)
					pass
			else:
				self.islistening=True
		if self.islistening==True and self.runstate>0:
			while self.runstate>0:
				client,addr=s.accept()
				if self.callback is not None:
					self.callback(client, addr)
		self.quit(s)
	
	def quit(self, s=False):
		self.runstate=0
		try:
			s.close()
		except:
			pass


class NetioClientTh:
	def __init__(self, address, timeout=None):
		self.conn=address
		self.timeout=timeout
		self.runstate=-1
	
	def close(self):
		self.quit()
		self.runstate=0
	
	def create(self):
		s=False
		if self.runstate!=0:
			s=socket.socket(self.conn[0])
			s.settimeout(self.timeout)
			s.connect(self.conn[1])
			self.runstate=1
		self.s=s
		return self.s
	
	def quit(self, s=False):
		self.runstate=0
		try:
			self.s.close()
		except:
			pass

class NetioClientObj:
	def __init__(self):
		self.rx=None
		self.tx=None



class NetioRxTh(threading.Thread):
	def __init__(self, cid, csocket, callback=None, length=256, timeout=None):
		self.cid=cid
		self.conn=csocket
		self.callback=callback
		self.timeout=timeout
		self.length=length
		self.d=NetioData()
		self.runstate=-1
		threading.Thread.__init__(self)
	
	def close(self):
		self.runstate=0
	
	def run(self):
		self.runstate=1
		while self.runstate>0:
			rx=dict({'cid':self.cid})
			self.conn.settimeout(self.timeout)
			rxdata=self.d.recv(self.conn, self.length)
			if rxdata!='':
				exec('rx.update('+rxdata+')')
				#rx.update(eval(rxdata))
				self.answer(self.callback(rx))
		self.quit()
	
	def answer(self, txdata):
		tx=dict({'return_code':404})
		tx.update(txdata)
		self.d.send(self.conn, str(tx))
	
	def quit(self):
		self.runstate=0
		try:
			self.conn.close()
		except:
			pass


class NetioTxTh:
	def __init__(self, cid, csocket, length=256, timeout=None):
		self.cid=cid
		self.conn=csocket
		self.timeout=timeout
		self.length=length
		self.queue=None
		self.d=NetioData()
		self.runstate=1
	
	def close(self):
		self.quit()
		self.runstate=0
	
	def send(self, txdata):
		while self.queue is not None:
			time.sleep(0.05)
		self.queue=txdata
		return self.txqueue()
	
	def txqueue(self):
		ret=dict({'return_code':404})
		if self.runstate>0:
			tx=dict({'cid':self.cid})
			try:
				self.conn.settimeout(self.timeout)
				tx.update(self.queue)
				self.d.send(self.conn, str(tx))
				ret=self.answer()
			except:
				ret=dict({'return_code':408})
				pass
		self.queue=None
		return ret
	
	def answer(self):
		rx=dict({'return_code':404})
		rxdata=self.d.recv(self.conn, self.length)
		if rxdata!='':
			exec('rx.update('+rxdata+')')
			#rx.update(eval(rxdata))
		return rx
	
	def quit(self):
		try:
			self.send(dict({'cmd':'close'}))
			self.conn.close()
		except:
			pass
