import os
import json
import time
from lubase import LU

class PortFs:
	RETURN_DEFAULT='default'
	RETURN_STRING='str'
	RETURN_INTEGER='int'
	RETURN_BOOLEAN='bool'
	IFSET_OVERWRITE='force'
	IFSET_WAIT='wait'
	IFSET_OMIT='omit'
	ANSWER_TIMEOUT=30
	
	lu=LU()
	cfg=dict({'fsRoot':'/var/portfs', 'sizeRoot':'32k', 'fsname':'', 'size':'128k', 'isolate':1, 'outstate':1, 'ext':'val'})
	
	def __init__(self, fsname, args=dict({}), common_returns='default'):
		try:
			self.cfg.update(json.loads(self.lu.loadFile('/etc/portfs.json')))
		except:
			pass
		self.cfg.update(args)
		self.cfg.update({'fsname':fsname})
		self.rettype=common_returns
		self.fext='.'+self.cfg['ext']
		self.create()
	
	
	def create(self):
		self.fsdir=self.cfg['fsRoot']+'/'+self.cfg['fsname']
		try:
			if self.isFs(True) is not True:
				os.system('mount -t tmpfs -o rw,size='+self.cfg['sizeRoot']+',defaults tmpfs '+self.cfg['fsRoot']+' > /dev/null 2>&1')
				self.lu.saveFile(self.cfg['fsRoot']+'/fs.is', '1')
			if self.isFs() is not True:
				os.mkdir(self.fsdir)
				if self.cfg['isolate']>0:
					os.system('mount -t tmpfs -o rw,size='+self.cfg['size']+',defaults tmpfs '+self.fsdir+' > /dev/null 2>&1')
				os.chmod(self.fsdir, 0o777)
				self.lu.saveFile(self.fsdir+'/fs.is', '1')
			return self.fsdir
		except:
			pass
			return self.getReturn(False)
	
	def destroy(self):
		try:
			os.system('rm -rf '+self.fsdir+'/* > /dev/null 2>&1')
			if self.cfg['isolate']>0:
				os.system('umount '+self.fsdir+' > /dev/null 2>&1')
			os.system('rm -rf '+self.fsdir+' > /dev/null 2>&1')
			return self.getReturn(True)
		except:
			pass
			return self.getReturn(False)
	
	
	def set(self, name, value, ifset='force'):
		if str(name)!='':
			allow=True
			if ifset=='omit':
				if self.isset(name, self.RETURN_BOOLEAN)==True:
					allow=False
			elif ifset=='wait':
				while self.isset(name, self.RETURN_BOOLEAN)==True:
					time.sleep(0.1)
			
			if allow==True:
				try:
					var=open(self.fsdir+'/'+str(name)+self.fext, 'w')
					var.write(value)
					var.close()
					os.chmod(self.fsdir+'/'+str(name)+self.fext, 0o666)
					return self.getReturn(True)
				except:
					pass
					return self.getReturn(False)
			else:
				return self.getReturn(False)
		else:
			return self.getReturn(False)
	
	def add(self, name, value):
		value=self.get(name)+value
		return self.set(name, value)
	
	
	def get(self, name, keep_lines=False, auto_unset=False):
		if str(name)!='':
			try:
				var=open(self.fsdir+'/'+str(name)+self.fext, 'r')
				value=var.read()
				var.close()
				if keep_lines!=True:
					value=str(value).replace("\n", '')
				if auto_unset==True:
					self.unset(name)
			except:
				pass
				value=''
			return str(value)
		else:
			return self.getReturn(False)
	
	
	def isset(self, name, rettype=None):
		if str(name)!='':
			try:
				var=open(self.fsdir+'/'+str(name)+self.fext, 'r')
				var.close()
				return self.getReturn(True, rettype)
			except:
				pass
				return self.getReturn(False, rettype)
		else:
			return self.getReturn(False, rettype)
	
	def unset(self, name):
		if str(name)!='':
			self.set(name, '')
			try:
				os.remove(self.fsdir+'/'+str(name)+self.fext)
				return self.getReturn(True)
			except:
				pass
				return self.getReturn(False)
			return value
		else:
			return self.getReturn(False)
	
	
	def sck(self, tx_sock, rx_sock, rx_callback=None, answer_timeout=30, answering_disabled=False):
		self.txs=tx_sock
		self.rxs=rx_sock
		if rx_callback is not None:
			self.rxc=rx_callback
		else:
			self.rxc=self.defCallback
		self.ANSWER_TIMEOUT=answer_timeout
		self.answoff=answering_disabled
	
	def tx(self, value):
		self.set(self.txs+'-tx', str(value), ifset=self.IFSET_WAIT)
		if self.answoff!=True:
			wait=0
			answ=''
			while wait<self.ANSWER_TIMEOUT*10:
				answ=self.get(self.txs+'-rx', auto_unset=True)
				if answ!='':
					wait=self.ANSWER_TIMEOUT*100
				else:
					wait=wait+1
					time.sleep(0.1)
			if answ=='':
				answ=None
		else:
			answ=None
		return answ
	
	def rx(self):
		ret=''
		recv=self.get(self.rxs+'-tx', auto_unset=True)
		if recv!='':
			ret=self.rxc(recv)
			if self.answoff!=True:
				self.set(self.rxs+'-rx', str(ret), ifset=self.IFSET_WAIT)
		return ret
	
	def defCallback(self, data):
		return False
	
	
	def getReturn(self, ret, rettype=None):
		if rettype is None:
			rettype=self.rettype
		if rettype=='default':
			return ret
		else:
			if ret==True:
				if rettype=='str':
					return '1'
				elif rettype=='int':
					return 1
				elif rettype=='bool':
					return True
			else:
				if rettype=='str':
					return '0'
				elif rettype=='int':
					return 0
				elif rettype=='bool':
					return False
	
	def isFs(self, fsRoot=False):
		if fsRoot is True:
			fsfile=self.cfg['fsRoot']+'/fs.is'
		else:
			fsfile=self.fsdir+'/fs.is'
		fsexists=self.lu.loadFile(fsfile, True)
		if fsexists is not False:
			if int(fsexists)>0:
				return True
			else:
				return False
		else:
			return False
