import os
import sys
import time
import signal
import select
import socket
import pybonjour
import threading


class BonjourRegister():
	def __init__(self, name, regtype, port):
		self.name=name
		self.regtype=regtype
		self.port=port
	
	def register(self):
		self.sdRef=pybonjour.DNSServiceRegister(name=self.name, regtype=self.regtype, port=self.port)
		return self.sdRef
	
	def unregister(self):
		self.sdRef.close()
		return True


class BonjourResolve():
	def __init__(self):
		self.timeout=5
		self.maxduration=180
		self.findhost=['', '']
		
		self.queried=[]
		self.resolved=[]
		self.catched=''
	
	
	def browse(self, regtype):
		browse_sdRef = pybonjour.DNSServiceBrowse(regtype = regtype, callBack = self.browse_callback)
		
		duration=0
		try:
			while True and duration<self.maxduration and self.catched=='':
				ready = select.select([browse_sdRef], [], [])
				if browse_sdRef in ready[0]:
					pybonjour.DNSServiceProcessResult(browse_sdRef)
				duration=duration+1
				time.sleep(1)
		finally:
			browse_sdRef.close()
	
	def find(self, regtype, namepart='', hostpart=''):
		self.findhost=[namepart, hostpart]
		self.browse(regtype)
		return self.catched
	
	
	def query_record_callback(self, sdRef, flags, interfaceIndex, errorCode, fullname, rrtype, rrclass, rdata, ttl):
		if errorCode == pybonjour.kDNSServiceErr_NoError:
			self.catched=str(socket.inet_ntoa(rdata))
			self.queried.append(True)
	
	
	def resolve_callback(self, sdRef, flags, interfaceIndex, errorCode, fullname, hosttarget, port, txtRecord):
		if errorCode != pybonjour.kDNSServiceErr_NoError:
			return
		
		trycatch=0
		if self.findhost[0]=='' and self.findhost[1]=='':
			trycatch=1
		else:
			if self.findhost[0]!='' and str(fullname).find(self.findhost[0])>-1:
				trycatch=1
			if self.findhost[1]!='' and str(hosttarget).find(self.findhost[1])>-1:
				trycatch=1
		
		if trycatch==1:
			query_sdRef = pybonjour.DNSServiceQueryRecord(interfaceIndex = interfaceIndex, fullname = hosttarget, rrtype = pybonjour.kDNSServiceType_A, callBack = self.query_record_callback)
			try:
				while not self.queried:
					ready = select.select([query_sdRef], [], [], self.timeout)
					if query_sdRef not in ready[0]:
						break
					pybonjour.DNSServiceProcessResult(query_sdRef)
				else:
					self.queried.pop()
			finally:
				query_sdRef.close()
		
		self.resolved.append(True)
	
	def browse_callback(self, sdRef, flags, interfaceIndex, errorCode, serviceName, regtype, replyDomain):
		if errorCode != pybonjour.kDNSServiceErr_NoError:
			return
		
		if not (flags & pybonjour.kDNSServiceFlagsAdd):
			#service removed
			return
		
		resolve_sdRef = pybonjour.DNSServiceResolve(0, interfaceIndex, serviceName, regtype, replyDomain, self.resolve_callback)
		try:
			while not self.resolved:
				ready = select.select([resolve_sdRef], [], [], self.timeout)
				if resolve_sdRef not in ready[0]:
					break
				pybonjour.DNSServiceProcessResult(resolve_sdRef)
			else:
				self.resolved.pop()
		finally:
			resolve_sdRef.close()
