#!/usr/bin/env python
# -*- coding: utf-8 -*-

import gi
gi.require_version('Gtk', '3.0')
from gi.repository import Gtk
from gi.repository import GObject


class  GtkApplication:
	def __init__(self, iface_xml=''):
		self.gui = Gtk
		self.gui.builder = self.gui.Builder()
		if iface_xml!='':
			self.importIface(iface_xml)
		self.handlers=dict({})
	
	def importIface(self, xml_file):
		self.gui.builder.add_from_file(xml_file)
	
	def start(self):
		self.gui.main()
	
	def quit(self, *args):
		self.gui.main_quit(*args)
	
	def handleSignals(self, signals, action):
		sig_handlers={}
		for signal in signals:
			sig_handlers[signal]=action
		self.handlers.update(sig_handlers)
	
	def connectSignals(self, sig_handlers={}):
		self.handlers.update(sig_handlers)
		self.gui.builder.connect_signals(self.handlers)
	
	def newWindow(self, window_id, onCreate_show=True, onDestroy_quit=True):
		window=self.gui.builder.get_object(window_id)
		if onCreate_show==True:
			window.show_all()
		if onDestroy_quit==True:
			window.connect('destroy', self.quit)
		return window
	
	def newTimer(self, interval, action):
		GObject.timeout_add(interval, action)
	
	def obj(self, object_name):
		return self.gui.builder.get_object(object_name)
	
	def get(self, objects, propert):
		if isinstance(objects, list)==False:
			return self.obj(objects).get_property(propert)
		else:
			ret=dict({})
			i=0
			while i<len(objects):
				ret.update({objects[i]:self.obj(objects[i]).get_property(propert)})
				i=i+1
			return ret
	
	def set(self, objects, propert, value):
		if isinstance(objects, list)==False:
			objects=[objects]
		i=0
		while i<len(objects):
			self.obj(objects[i]).set_property(propert, value)
			i=i+1
		return True



class DynamicGlade:
	def __init__(self, locales='en_EN', dir_tpl='.', dir_tmp='/tmp', dir_lng='.'):
		self.lng=locales
		self.dirs=dict({"tpl":dir_tpl, "tmp":dir_tmp, "lng":dir_lng})
		from lubase import LU
		self.c=LU(self.dirs['tmp'])
		self.haslocales=False
	
	def create(self, template):
		self.getLocales(template)
		content=self.c.loadFile(self.dirs['tpl']+'/'+template+'.glade')
		terms=list(self.terms.items())
		for term in terms:
			newterm=str(term[1])
			newterm=newterm.replace('<br>', "\n")
			newterm=newterm.replace('<br/>', "\n")
			content=content.replace('%['+str(term[0])+']%', newterm)
		save=self.c.saveFile(self.dirs['tmp']+'/'+template+'-'+self.lng+'.glade', content)
		if save==False:
			return False
		else:
			return self.dirs['tmp']+'/'+template+'-'+self.lng+'.glade'
	
	def release(self, path='', clean_all=False):
		if clean_all==True:
			path=self.dirs['tmp']+'/*.glade'
		if path!='':
			return self.c.rmFile(path)
		else:
			return False
	
	def flush(self):
		return self.release(clean_all=True)
	
	def getLocales(self, addcustom=''):
		if self.haslocales==False:
			self.defterms=eval('dict({'+self.c.loadFile(self.dirs['lng']+'/'+self.lng+'.lng')+'})')
			self.haslocales=True
		self.terms=self.defterms
		if addcustom!='':
			custterms=self.c.loadFile(self.dirs['lng']+'/'+addcustom+'-'+self.lng+'.lng')
			if custterms!=False:
				self.terms.update(eval('{'+custterms+'}'))
		return self.terms



class CommonWindows:
	def __init__(self, gtk_application, iface_xml='/usr/share/lunet/gtkapp.glade'):
		self.app=gtk_application
		self.app.importIface(iface_xml)
		self.app.handleSignals(['MSGreturn'], self.sigMsgReturn)
		self.app.handleSignals(['ABTreturn'], self.sigAbtReturn)
		
		self.w=dict({'message':self.app.newWindow('winMessage', False, False),\
			     'about':self.app.newWindow('winAbout', False, False)})
		self.callback=dict({'message':[self.doNothing, dict({})],\
				    'about':[self.doNothing, dict({})]})
	
	def show(self, window, args=dict({}), callback=False):
		self.w[window].show_all()
		if window=='message':
			self.showMsg(args)
		elif window=='about':
			self.showAbt(args)
		self.callback[window][1]=args
		if bool(callback)!=False:
			self.callback[window][0]=callback
	
	
	def showMsg(self, largs):
		args=dict({'parent':'winMain', 'severity':'info', 'title':'Message', 'text':'', 'more':'more...', 'details':'', 'actions':['ok']})
		args.update(largs)
		if args['severity']=='yesno':
			args['actions']=['yes', 'no']
		
		self.app.set('winMessage', 'transient_for', self.app.obj(args['parent']))
		self.app.set(['MSGokAct', 'MSGcancelAct', 'MSGyesAct', 'MSGnoAct', 'MSGrefreshAct', 'MSGstopAct', 'MSGcloseAct', 'MSGquitAct'], 'visible', False)
		
		self.app.set('winMessage', 'icon_name', self.getMsgIcon(args['severity']))
		self.app.set('MSGicon', 'stock', self.getMsgIcon(args['severity']))
		self.app.set('winMessage', 'title', args['title'])
		self.app.set('MSGtext', 'label', args['text'])
		self.app.set('MSGdetailsHead', 'label', args['more'])
		self.app.set('MSGdetailsText', 'text', args['details'])
		if args['details']!='':
			self.app.set('MSGdetailsExpand', 'visible', True)
		i=0
		while i<len(args['actions']):
			self.app.set('MSG'+args['actions'][i]+'Act', 'visible', True)
			i=i+1
	def getMsgIcon(self, severity):
		icon='gtk-'+severity
		icon=icon.replace('info', 'dialog-info')
		icon=icon.replace('debug', 'dialog-info')
		icon=icon.replace('notice', 'dialog-info')
		icon=icon.replace('warn', 'dialog-warning')
		icon=icon.replace('err', 'dialog-error')
		icon=icon.replace('crit', 'dialog-error')
		icon=icon.replace('alert', 'stop')
		icon=icon.replace('emerg', 'stop')
		icon=icon.replace('yesno', 'dialog-question')
		return icon
	def sigMsgReturn(self, widget, *args):
		ret=widget.get_property('label')
		self.app.obj('winMessage').hide()
		self.callback['message'][0](ret.replace('gtk-', ''), self.callback['message'][1])
	
	
	def showAbt(self, largs):
		args=dict({'parent':'winMain', 'title':'About', 'image':'', 'name':'', 'version':'', 'description':'', 'copyright':'', 'links':[], 'info':''})
		args.update(largs)
		
		self.app.set('winAbout', 'transient_for', self.app.obj(args['parent']))
		self.app.set(['ABTlink1', 'ABTlink2', 'ABTlink3'], 'visible', False)
		
		self.app.set('winAbout', 'title', args['title'])
		self.app.obj('ABTimage').set_from_file(args['image'])
		self.app.set('ABTtxtName', 'label', args['name'])
		self.app.set('ABTtxtVer', 'label', args['version'])
		self.app.set('ABTtxtDesc', 'label', args['description'])
		self.app.set('ABTtxtCopy', 'label', args['copyright'])
		i=0
		lcnt=len(args['links'])
		if lcnt>3:
			lcnt=3
		while i<lcnt:
			self.app.set('ABTlink'+str(i+1), 'visible', True)
			self.app.set('ABTlink'+str(i+1), 'label', args['links'][i][0])
			self.app.set('ABTlink'+str(i+1), 'uri', args['links'][i][1])
			i=i+1
		self.app.set('ABTinfoText', 'text', args['info'])
		if args['info']!='':
			self.app.set(['ABTinfoExpand'], 'visible', True)
	def sigAbtReturn(self, widget, *args):
		self.app.obj('winAbout').hide()
		self.callback['about'][0](True, self.callback['about'][1])
	
	
	def doNothing(self, response, args):
		return response


try:
	import gtk_webapp
	class WebAppClient(gtk_webapp.WebAppClient):
		def dummy(self):
			return False
except:
	pass
