#!/usr/bin/env python

##################################################
## Common and basic wide-used functions         ##
##################################################

import os
import random
import json
from datetime import datetime


class LU:
	def __init__(self, tmpdir='/tmp'):
		self.tmpdir = tmpdir
	
	
	def getCmdArgs(self, cargs, pattern):
		# TODO
		return True
	
	
	def loadFile(self, name, single_line=False):
		if name!='':
			try:
				f=open(name, 'r')
				out=f.read()
				f.close()
				if single_line==True:
					out=out.replace("\n", '')
				return out
			except:
				pass
				return False
		else:
			return False
	
	def saveFile(self, name, content):
		if name!='':
			try:
				f=open(name, 'w')
				f.write(content)
				f.close()
				return True
			except:
				pass
				return False
		else:
			return False
	
	def rmFile(self, name):
		if name!='':
			try:
				os.remove(name)
				return True
			except:
				pass
				return False
		else:
			return False
	
	
	def execCmd(self, command, single_line=False, quiet=False):
		ret=dict({"status":False, "stdout":"", "stderr":""})
		if command!='':
			if quiet==False:
				outid=str(random.randint(100000, 999999))
				fout=self.tmpdir+'/'+outid+'.stdout'
				ferr=self.tmpdir+'/'+outid+'.stderr'
			else:
				fout='/dev/null'
				ferr='/dev/null'
			try:
				os.system(command+' > '+fout+' 2>'+ferr)
				if quiet==False:
					ret['stdout']=self.loadFile(fout)
					ret['stderr']=self.loadFile(ferr)
					self.rmFile(fout)
					self.rmFile(ferr)
					if single_line==True:
						ret['stdout']=ret['stdout'].replace("\n", '')
					if ret['stdout'].replace("\n", '')=='':
						if ret['stderr'].replace("\n", '')!='':
							ret['status']=False
						else:
							ret['status']=True
					else:
						ret['status']=True
				else:
					ret['status']=True
				return ret
			except:
				pass
				return ret
		else:
			return ret



class ConfConstructor:
	def __init__(self, cfgdir):
		self.cfgdir = cfgdir
	
	def load(self, name):
		values={}
		if name!='':
			try:
				f=open(self.cfgdir+'/'+name, 'r')
				lines=f.read()
				f.close()
				line=lines.split("\n")
				for item in line:
					val=item.split('=')
					if val[0]!='' and len(val)>1:
						value=val[1].replace('"', '')
						values[val[0]]=value
			except:
				pass
			return values
		else:
			return values
	
	def save(self, name, values):
		if name!='':
			out='#!/bin/bash'
			out=out+"\n\n"
			try:
				items=list(values.items())
				for item in items:
					out=out+item[0]+'="'+item[1]+'"'+"\n"
				f=open(self.cfgdir+'/'+name, 'w')
				f.write(out)
				f.close()
				return True
			except:
				pass
				return False
		else:
			return False



class LogIt:
	def __init__(self, process, pid=0, facility='user'):
		self.process=process
		self.pid=pid
		self.facility=self.checkFacility(facility)
		
		self.lu=LU()
		self.cfg=dict(json.loads(self.lu.loadFile('/etc/logit.json')))
		
		self.pname=self.process
		if self.pid>0:
			self.pname=self.process+'['+str(self.pid)+']'
	
	
	def set(self, severity, message, args=dict({}), callback=False):
		severity=self.checkSeverity(severity)
		outs=self.getOutputs(severity)
		msgline=str(self.getDatetime())+' ('+self.facility+'.'+severity+') '+self.pname+': '+message
		for outcmd in outs:
			if outcmd=='file':
				os.system('echo "'+msgline+'" >> '+self.cfg['logdir']+'/'+self.process+'.log')
			if outcmd=='syslog':
				os.system('logger -t '+self.pname+' -p '+self.facility+'.'+severity+' "'+message+'"')
			if outcmd=='stdout' or outcmd=='stderr':
				cmd='echo "'+msgline+'"'
				if outcmd=='stderr':
					cmd=cmd+' >&2'
				os.system(cmd)
				pass
		
		if bool(callback)!=False:
			callback(severity, message, args)
	
	
	def checkFacility(self, facility):
		return self.checkLevel('fac', facility)
	def checkSeverity(self, severity):
		return self.checkLevel('sev', severity)
	
	def checkLevel(self, ltype, lval):
		if isinstance(lval, int):
			levels=dict({'fac':['kern', 'user', '', 'daemon', '',  'auth', 'authpriv', '', '', '', 'cron'], 'sev':['emerg', 'alert', 'crit', 'err', 'warning', 'notice', 'info', 'debug']})
			ret=levels[ltype][lval]
		else:
			if ltype=='fac':
				lval=lval.replace('security', 'auth')
			elif ltype=='sev':
				lval=lval.replace('panic', 'emerg')
				lval=lval.replace('error', 'err')
				lval=lval.replace('warn', 'warning')
			ret=lval
		return ret
	
	def getOutputs(self, severity):
		ret=[]
		if severity in self.cfg['output']:
			if self.facility in self.cfg['output'][severity]:
				ret=self.cfg['output'][severity][self.facility]
			else:
				if self.facility in self.cfg['output']['DEFAULT']:
					ret=self.cfg['output']['DEFAULT'][self.facility]
				else:
					ret=self.cfg['output']['DEFAULT']['DEFAULT']
		else:
			if self.facility in self.cfg['output']['DEFAULT']:
				ret=self.cfg['output']['DEFAULT'][self.facility]
			else:
				ret=self.cfg['output']['DEFAULT']['DEFAULT']
		return ret
	
	def getDatetime(self):
		return datetime.now().strftime("%Y-%m-%d %H:%M:%S")
	
	
	
